/**
 * For copyright information see the LICENSE document.
 */

package gwlpr.protocol.handshake.messages;

import gwlpr.protocol.serialization.GWMessage;
import static gwlpr.protocol.serialization.GWMessageSerializationRegistry.getFilter;
import static gwlpr.protocol.serialization.GWMessageSerializationRegistry.register;
import gwlpr.protocol.serialization.NettySerializationFilter;
import gwlpr.protocol.util.IsArray;
import io.netty.buffer.ByteBuf;


/**
 * Generated by hand :P
 *
 * STEP 2: FOR LS AND GS
 *
 * Special message: Includes header and length in the first two bytes.
 *
 * @author _rusty
 */
public class P000_ClientSeed
{

    public static class Header extends GWMessage
    {
        public short Header;
        public short Length;

        @Override
        public short getHeader() { return 0; }
        public static int getLength() { return 2; }
    }


    public static class Payload extends GWMessage
    {
        @IsArray(constant = true, size = 64, prefixLength = -1)
        public byte[] ClientPublicKey;

        @Override
        public short getHeader() { return 0; }
        public static int getLength() { return 64; }
    }


    private static final NettySerializationFilter headFilter;
    private static final NettySerializationFilter payloadFilter;


    static
    {
        register(Header.class);
        register(Payload.class);

        headFilter = getFilter(Header.class);
        payloadFilter = getFilter(Payload.class);
    }


    public static int getLength()
    {
        return Header.getLength() + Payload.getLength();
    }


    public static boolean check(Header header)
    {
        return (header.Header == 0 && header.Length == (Header.getLength() + Payload.getLength()));
    }


    public static Header serializeHeader(ByteBuf buffer)
    {
        // failcheck
        if (buffer.readableBytes() < Header.getLength()) { return null; }

        Header result = new Header();
        headFilter.deserialize(buffer, result);

        return result;
    }


    public static Payload serializePayload(ByteBuf buffer)
    {
        // failcheck
        if (buffer.readableBytes() < Payload.getLength()) { return null; }

        Payload result = new Payload();
        payloadFilter.deserialize(buffer, result);

        return result;
    }
}
